@@ -1,9 +1,8 @@
 # -*- test-case-name: twisted.conch.test.test_filetransfer -*-
 #
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-#
 
 import struct, errno
 
@@ -15,6 +14,8 @@ from twisted.conch.interfaces import ISFTPServer, ISFTPFile
 
 from zope import interface
 
+
+
 class FileTransferBase(protocol.Protocol):
 
     versions = (3, )
@@ -79,7 +80,7 @@ class FileTransferBase(protocol.Protocol):
             attrs['mtime'] = mtime
             data = data[8:]
         if flags & FILEXFER_ATTR_EXTENDED == FILEXFER_ATTR_EXTENDED:
-            extended_count ,= struct.unpack('!L', data[4:])
+            extended_count ,= struct.unpack('!L', data[:4])
             data = data[4:]
             for i in xrange(extended_count):
                 extended_type, data = getNS(data)
@@ -309,7 +310,7 @@ class FileTransferServer(FileTransferBase):
             data += NS(filename)
             data += NS(longname)
             data += self._packAttributes(attrs)
-        self.sendPacket(FXP_NAME, requestId + 
+        self.sendPacket(FXP_NAME, requestId +
                         struct.pack('!L', len(result))+data)
 
     def packet_STAT(self, data, followLinks = 1):
@@ -330,7 +331,7 @@ class FileTransferServer(FileTransferBase):
         handle, data = getNS(data)
         assert data == '', 'still have data in FSTAT: %s' % repr(data)
         if handle not in self.openFiles:
-            self._ebStatus(failure.Failure(KeyError('%s not in self.openFiles' 
+            self._ebStatus(failure.Failure(KeyError('%s not in self.openFiles'
                                         % handle)), requestId)
         else:
             fileObj = self.openFiles[handle]
@@ -421,6 +422,8 @@ class FileTransferServer(FileTransferBase):
             elif reason.value.errno == errno.EACCES: # permission denied
                 code = FX_PERMISSION_DENIED
                 message = reason.value.strerror
+            elif reason.value.errno == errno.EEXIST:
+                code = FX_FILE_ALREADY_EXISTS
             else:
                 log.err(reason)
         elif reason.type == EOFError: # EOF
@@ -447,11 +450,25 @@ class FileTransferServer(FileTransferBase):
         data += NS(lang)
         self.sendPacket(FXP_STATUS, data)
 
+
+    def connectionLost(self, reason):
+        """
+        Clean all opened files and directories.
+        """
+        for fileObj in self.openFiles.values():
+            fileObj.close()
+        self.openFiles = {}
+        for (dirObj, dirIter) in self.openDirs.values():
+            dirObj.close()
+        self.openDirs = {}
+
+
+
 class FileTransferClient(FileTransferBase):
 
     def __init__(self, extData = {}):
         """
-        extData is a dict of extended_name : extended_data items
+        @param extData: a dict of extended_name : extended_data items
         to be sent to the server.
         """
         FileTransferBase.__init__(self)
@@ -484,12 +501,15 @@ class FileTransferClient(FileTransferBase):
         """
         Open a file.
 
-        filename is a string representing the file to open.
+        This method returns a L{Deferred} that is called back with an object
+        that provides the L{ISFTPFile} interface.
 
-        flags is a integer of the flags to open the file with, ORed together.
+        @param filename: a string representing the file to open.
+
+        @param flags: a integer of the flags to open the file with, ORed together.
         The flags and their values are listed at the bottom of this file.
 
-        attrs is a list of attributes to open the file with.  It is a
+        @param attrs: a list of attributes to open the file with.  It is a
         dictionary, consisting of 0 or more keys.  The possible keys are::
 
             size: the size of the file in bytes
@@ -504,9 +524,6 @@ class FileTransferClient(FileTransferBase):
 
         NOTE: there is no way to indicate text or binary files.  it is up
         to the SFTP client to deal with this.
-
-        This method returns an Deferred that is called back with an object
-        that meets the ISFTPFile interface.
         """
         data = NS(filename) + struct.pack('!L', flags) + self._packAttributes(attrs)
         d = self._sendRequest(FXP_OPEN, data)
@@ -517,9 +534,9 @@ class FileTransferClient(FileTransferBase):
         """
         Remove the given file.
 
-        filename is the name of the file as a string.
-
         This method returns a Deferred that is called back when it succeeds.
+
+        @param filename: the name of the file as a string.
         """
         return self._sendRequest(FXP_REMOVE, NS(filename))
 
@@ -527,10 +544,10 @@ class FileTransferClient(FileTransferBase):
         """
         Rename the given file.
 
-        oldpath is the current location of the file.
-        newpath is the new file name.
-
         This method returns a Deferred that is called back when it succeeds.
+
+        @param oldpath: the current location of the file.
+        @param newpath: the new file name.
         """
         return self._sendRequest(FXP_RENAME, NS(oldpath)+NS(newpath))
 
@@ -538,12 +555,13 @@ class FileTransferClient(FileTransferBase):
         """
         Make a directory.
 
-        path is the name of the directory to create as a string.
-        attrs is a dictionary of attributes to create the directory with.
-        It's meaning is the same as the attrs in the openFile method.
-
-        This method returns a Deferred that is called back when it is 
+        This method returns a Deferred that is called back when it is
         created.
+
+        @param path: the name of the directory to create as a string.
+
+        @param attrs: a dictionary of attributes to create the directory
+        with.  Its meaning is the same as the attrs in the openFile method.
         """
         return self._sendRequest(FXP_MKDIR, NS(path)+self._packAttributes(attrs))
 
@@ -551,12 +569,12 @@ class FileTransferClient(FileTransferBase):
         """
         Remove a directory (non-recursively)
 
-        path is the directory to remove.
-
         It is an error to remove a directory that has files or directories in
         it.
 
         This method returns a Deferred that is called back when it is removed.
+
+        @param path: the directory to remove.
         """
         return self._sendRequest(FXP_RMDIR, NS(path))
 
@@ -564,17 +582,15 @@ class FileTransferClient(FileTransferBase):
         """
         Open a directory for scanning.
 
-        path is the directory to open.
-
-        This method returns a Deferred that is called back with an iterable 
-        object that has a close() method,
+        This method returns a Deferred that is called back with an iterable
+        object that has a close() method.
 
         The close() method is called when the client is finished reading
         from the directory.  At this point, the iterable will no longer
         be used.
 
         The iterable returns triples of the form (filename, longname, attrs)
-        or a Deferred that returns the same.  The sequence must support 
+        or a Deferred that returns the same.  The sequence must support
         __getitem__, but otherwise may be any 'sequence-like' object.
 
         filename is the name of the file relative to the directory.
@@ -588,6 +604,8 @@ class FileTransferClient(FileTransferBase):
         size in bytes, modification time.
 
         attrs is a dictionary in the format of the attrs argument to openFile.
+
+        @param path: the directory to open.
         """
         d = self._sendRequest(FXP_OPENDIR, NS(path))
         self.wasAFile[d] = (0, path)
@@ -597,13 +615,13 @@ class FileTransferClient(FileTransferBase):
         """
         Return the attributes for the given path.
 
-        path is the path to return attributes for as a string.
-        followLinks is a boolean.  if it is True, follow symbolic links
-        and return attributes for the real path at the base.  if it is False,
-        return attributes for the specified path.
-
         This method returns a dictionary in the same format as the attrs
         argument to openFile or a Deferred that is called back with same.
+
+        @param path: the path to return attributes for as a string.
+        @param followLinks: a boolean.  if it is True, follow symbolic links
+        and return attributes for the real path at the base.  if it is False,
+        return attributes for the specified path.
         """
         if followLinks: m = FXP_STAT
         else: m = FXP_LSTAT
@@ -613,12 +631,12 @@ class FileTransferClient(FileTransferBase):
         """
         Set the attributes for the path.
 
-        path is the path to set attributes for as a string.
-        attrs is a idctionary in the same format as the attrs argument to
-        openFile.
-
         This method returns when the attributes are set or a Deferred that is
         called back when they are.
+
+        @param path: the path to set attributes for as a string.
+        @param attrs: a dictionary in the same format as the attrs argument to
+        openFile.
         """
         data = NS(path) + self._packAttributes(attrs)
         return self._sendRequest(FXP_SETSTAT, data)
@@ -627,10 +645,10 @@ class FileTransferClient(FileTransferBase):
         """
         Find the root of a set of symbolic links.
 
-        path is the path of the symlink to read.
-
         This method returns the target of the link, or a Deferred that
         returns the same.
+
+        @param path: the path of the symlink to read.
         """
         d = self._sendRequest(FXP_READLINK, NS(path))
         return d.addCallback(self._cbRealPath)
@@ -639,11 +657,11 @@ class FileTransferClient(FileTransferBase):
         """
         Create a symbolic link.
 
-        linkPath is is the pathname of the symlink as a string
-        targetPath is the path of the target of the link as a string.
-
         This method returns when the link is made, or a Deferred that
         returns the same.
+
+        @param linkPath: the pathname of the symlink as a string
+        @param targetPath: the path of the target of the link as a string.
         """
         return self._sendRequest(FXP_SYMLINK, NS(linkPath)+NS(targetPath))
 
@@ -651,10 +669,10 @@ class FileTransferClient(FileTransferBase):
         """
         Convert any path to an absolute path.
 
-        path is the path to convert as a string.
-
         This method returns the absolute path as a string, or a Deferred
         that returns the same.
+
+        @param path: the path to convert as a string.
         """
         d = self._sendRequest(FXP_REALPATH, NS(path))
         return d.addCallback(self._cbRealPath)
@@ -666,11 +684,12 @@ class FileTransferClient(FileTransferBase):
     def extendedRequest(self, request, data):
         """
         Make an extended request of the server.
-        request is the name of the extended request to make.
-        data is any other data that goes along with the request.
 
         The method returns a Deferred that is called back with
         the result of the extended request.
+
+        @param request: the name of the extended request to make.
+        @param data: any other data that goes along with the request.
         """
         return self._sendRequest(FXP_EXTENDED, NS(request) + data)
 
@@ -738,9 +757,9 @@ class FileTransferClient(FileTransferBase):
         """
         Called when the client sends their version info.
 
-        otherVersion is an integer representing the version of the SFTP
+        @param otherVersion: an integer representing the version of the SFTP
         protocol they are claiming.
-        extData is a dictionary of extended_name : extended_data items.
+        @param extData: a dictionary of extended_name : extended_data items.
         These items are sent by the client to indicate additional features.
         """
 
@@ -812,9 +831,23 @@ class SFTPError(Exception):
     def __init__(self, errorCode, errorMessage, lang = ''):
         Exception.__init__(self)
         self.code = errorCode
-        self.message = errorMessage
+        self._message = errorMessage
         self.lang = lang
 
+
+    def message(self):
+        """
+        A string received over the network that explains the error to a human.
+        """
+        # Python 2.6 deprecates assigning to the 'message' attribute of an
+        # exception. We define this read-only property here in order to
+        # prevent the warning about deprecation while maintaining backwards
+        # compatibility with object clients that rely on the 'message'
+        # attribute being set correctly. See bug #3897.
+        return self._message
+    message = property(message)
+
+
     def __str__(self):
         return 'SFTPError %s: %s' % (self.code, self.message)
 
@@ -847,10 +880,11 @@ FXP_EXTENDED        = 200
 FXP_EXTENDED_REPLY  = 201
 
 FILEXFER_ATTR_SIZE        = 0x00000001
-FILEXFER_ATTR_OWNERGROUP  = 0x00000002
+FILEXFER_ATTR_UIDGID      = 0x00000002
+FILEXFER_ATTR_OWNERGROUP  = FILEXFER_ATTR_UIDGID
 FILEXFER_ATTR_PERMISSIONS = 0x00000004
-FILEXFER_ATTR_ACMODTIME   = 0x00000009
-FILEXFER_ATTR_EXTENDED    = 0x+80000000
+FILEXFER_ATTR_ACMODTIME   = 0x00000008
+FILEXFER_ATTR_EXTENDED    = 0x80000000L
 
 FILEXFER_TYPE_REGULAR        = 1
 FILEXFER_TYPE_DIRECTORY      = 2
@@ -875,11 +909,11 @@ FX_BAD_MESSAGE                 = 5
 FX_NO_CONNECTION               = 6
 FX_CONNECTION_LOST             = 7
 FX_OP_UNSUPPORTED              = 8
-# http://www.ietf.org/internet-drafts/draft-ietf-secsh-filexfer-12.txt defines
-# more useful error codes, but so far OpenSSH doesn't implement them.  We use
-# them internally for clarity, but for now define them all as FX_FAILURE to be
+FX_FILE_ALREADY_EXISTS         = 11
+# http://tools.ietf.org/wg/secsh/draft-ietf-secsh-filexfer/ defines more
+# useful error codes, but so far OpenSSH doesn't implement them.  We use them
+# internally for clarity, but for now define them all as FX_FAILURE to be
 # compatible with existing software.
-FX_FILE_ALREADY_EXISTS         = FX_FAILURE
 FX_NOT_A_DIRECTORY             = FX_FAILURE
 FX_FILE_IS_A_DIRECTORY         = FX_FAILURE
 
