@@ -1,24 +1,31 @@
 # -*- test-case-name: twisted.conch.test.test_ssh -*-
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
+import struct
+
 try:
-    import Crypto
+    import Crypto.Cipher.DES3
 except ImportError:
     Crypto = None
 
+try:
+    import pyasn1
+except ImportError:
+    pyasn1 = None
+
 from twisted.conch.ssh import common, session, forwarding
 from twisted.conch import avatar, error
+from twisted.conch.test.keydata import publicRSA_openssh, privateRSA_openssh
+from twisted.conch.test.keydata import publicDSA_openssh, privateDSA_openssh
 from twisted.cred import portal
 from twisted.internet import defer, protocol, reactor
 from twisted.internet.error import ProcessTerminated
-from twisted.protocols import loopback
 from twisted.python import failure, log
 from twisted.trial import unittest
 
 from test_recvline import LoopbackRelay
 
-import struct
 
 
 class ConchTestRealm:
@@ -71,7 +78,7 @@ class ConchTestAvatar(avatar.ConchUser):
         return 1
 
     def logout(self):
-        loggedOut = True
+        self.loggedOut = True
         for listener in self.listeners.values():
             log.msg('stopListening %s' % listener)
             listener.stopListening()
@@ -226,39 +233,26 @@ class SuperEchoTransport:
         self.proto.errConnectionLost()
         self.proto.processEnded(failure.Failure(ProcessTerminated(0, None, None)))
 
-class _LogTimeFormatMixin:
 
-    def setUpClass(self):
-        from twisted.python import log
-        self._oldTimeFormat = log.FileLogObserver.timeFormat
-        log.FileLogObserver.timeFormat = '%Y/%m/%d %H:%M:%S %Z'
-
-    def tearDownClass(self):
-        log.FileLogObserver.timeFormat = self._oldTimeFormat
-
-if Crypto: # stuff that needs PyCrypto to even import
+if Crypto is not None and pyasn1 is not None:
     from twisted.conch import checkers
     from twisted.conch.ssh import channel, connection, factory, keys
     from twisted.conch.ssh import transport, userauth
 
-    from test_keys import publicRSA_openssh, privateRSA_openssh
-    from test_keys import publicDSA_openssh, privateDSA_openssh
-
-
     class UtilityTestCase(unittest.TestCase):
         def testCounter(self):
             c = transport._Counter('\x00\x00', 2)
             for i in xrange(256 * 256):
-                self.assertEquals(c(), struct.pack('!H', i + 1))
+                self.assertEquals(c(), struct.pack('!H', (i + 1) % (2 ** 16)))
             # It should wrap around, too.
             for i in xrange(256 * 256):
-                self.assertEquals(c(), struct.pack('!H', i + 1))
+                self.assertEquals(c(), struct.pack('!H', (i + 1) % (2 ** 16)))
 
 
     class ConchTestPublicKeyChecker(checkers.SSHPublicKeyDatabase):
         def checkKey(self, credentials):
             unittest.assertEquals(credentials.username, 'testuser', 'bad username')
-            unittest.assertEquals(credentials.blob, keys.getPublicKeyString(data=publicDSA_openssh))
+            unittest.assertEquals(credentials.blob, keys.Key.fromString(publicDSA_openssh).blob())
             return 1
 
     class ConchTestPasswordChecker:
@@ -290,19 +284,23 @@ if Crypto: # stuff that needs PyCrypto to even import
             proto = ConchTestServer()
             proto.supportedPublicKeys = self.privateKeys.keys()
             proto.factory = self
+
+            if hasattr(self, 'expectedLoseConnection'):
+                proto.expectedLoseConnection = self.expectedLoseConnection
+
             self.proto = proto
             return proto
 
         def getPublicKeys(self):
             return {
-                'ssh-rsa':keys.getPublicKeyString(data=publicRSA_openssh),
-                'ssh-dss':keys.getPublicKeyString(data=publicDSA_openssh)
+                'ssh-rsa': keys.Key.fromString(publicRSA_openssh),
+                'ssh-dss': keys.Key.fromString(publicDSA_openssh)
             }
 
         def getPrivateKeys(self):
             return {
-                'ssh-rsa':keys.getPrivateKeyObject(data=privateRSA_openssh),
-                'ssh-dss':keys.getPrivateKeyObject(data=privateDSA_openssh)
+                'ssh-rsa': keys.Key.fromString(privateRSA_openssh),
+                'ssh-dss': keys.Key.fromString(privateDSA_openssh)
             }
 
         def getPrimes(self):
@@ -316,7 +314,6 @@ if Crypto: # stuff that needs PyCrypto to even import
     class ConchTestBase:
 
         done = 0
-        allowedToError = 0
 
         def connectionLost(self, reason):
             if self.done:
@@ -327,9 +324,15 @@ if Crypto: # stuff that needs PyCrypto to even import
 
         def receiveError(self, reasonCode, desc):
             self.expectedLoseConnection = 1
-            if not self.allowedToError:
-                unittest.fail('got disconnect for %s: reason %s, desc: %s' %
-                               (self, reasonCode, desc))
+            # Some versions of OpenSSH (for example, OpenSSH_5.3p1) will
+            # send a DISCONNECT_BY_APPLICATION error before closing the
+            # connection.  Other, older versions (for example,
+            # OpenSSH_5.1p1), won't.  So accept this particular error here,
+            # but no others.
+            unittest.assertEquals(
+                reasonCode, transport.DISCONNECT_BY_APPLICATION,
+                'got disconnect for %s: reason %s, desc: %s' % (
+                    self, reasonCode, desc))
             self.loseConnection()
 
         def receiveUnimplemented(self, seqID):
@@ -350,7 +353,7 @@ if Crypto: # stuff that needs PyCrypto to even import
             transport.SSHClientTransport.connectionLost(self, reason)
 
         def verifyHostKey(self, key, fp):
-            unittest.assertEquals(key, keys.getPublicKeyString(data = publicRSA_openssh))
+            unittest.assertEquals(key, keys.Key.fromString(publicRSA_openssh).blob())
             unittest.assertEquals(fp,'3d:13:5f:cb:c9:79:8a:93:06:27:65:bc:3d:0b:8f:af')
             return defer.succeed(1)
 
@@ -375,10 +378,10 @@ if Crypto: # stuff that needs PyCrypto to even import
 
         def getPrivateKey(self):
             self.canSucceedPublicKey = 1
-            return defer.succeed(keys.getPrivateKeyObject(data=privateDSA_openssh))
+            return defer.succeed(keys.Key.fromString(privateDSA_openssh))
 
         def getPublicKey(self):
-            return keys.getPublicKeyString(data=publicDSA_openssh)
+            return keys.Key.fromString(publicDSA_openssh)
 
     class ConchTestClientConnection(connection.SSHConnection):
 
@@ -412,7 +415,6 @@ if Crypto: # stuff that needs PyCrypto to even import
             good .... good
             """
             log.msg('unknown open failed')
-            log.flushErrors()
             self.conn.addResult()
 
         def channelOpen(self, ignored):
@@ -436,7 +438,6 @@ if Crypto: # stuff that needs PyCrypto to even import
 
         def _ebRequestWorked(self, ignored):
             log.msg('fail exec finished')
-            log.flushErrors()
             self.conn.addResult()
             self.loseConnection()
 
@@ -698,8 +699,14 @@ class SSHProtocolTestCase(unittest.TestCase):
     if not Crypto:
         skip = "can't run w/o PyCrypto"
 
-    def testOurServerOurClient(self):
-        """test the Conch server against the Conch client
+    if not pyasn1:
+        skip = "can't run w/o PyASN1"
+
+    def test_ourServerOurClient(self):
+        """
+        Run the Conch server against the Conch client.  Set up several different
+        channels which exercise different behaviors and wait for them to
+        complete.  Verify that the channels with errors log them.
         """
         realm = ConchTestRealm()
         p = portal.Portal(realm)
@@ -723,7 +730,21 @@ class SSHProtocolTestCase(unittest.TestCase):
                                 self.serverTransport.clearBuffer)
             log.callWithContext({'system': 'clientTransport'},
                                 self.clientTransport.clearBuffer)
-        self.failIf(self.server.done and self.client.done)
+        self.assertFalse(self.server.done and self.client.done)
+
+        errors = self.flushLoggedErrors(error.ConchError)
+        errors.sort(key=lambda reason: reason.value.args)
+
+        # Two errors exactly are expected.  Report the whole list if we get a
+        # different number.
+        self.assertEquals(
+            len(errors), 2, "Expected two errors, got: %r" % (errors,))
+
+        # SSHUnknownChannel causes this to be logged.
+        self.assertEquals(errors[0].value.args, (3, 'unknown channel'))
+        # SSHTestFailExecChannel causes this to be logged.
+        self.assertEquals(errors[1].value.args, ('bad exec', None))
+
 
 
 class TestSSHFactory(unittest.TestCase):
@@ -731,19 +752,138 @@ class TestSSHFactory(unittest.TestCase):
     if not Crypto:
         skip = "can't run w/o PyCrypto"
 
-    def testMultipleFactories(self):
-        f1 = factory.SSHFactory()
-        f2 = factory.SSHFactory()
-        gpk = lambda: {'ssh-rsa' : "don't use"}
-        f1.getPrimes = lambda: None
-        f2.getPrimes = lambda: {1:(2,3)}
-        f1.getPublicKeys = f2.getPublicKeys = gpk
-        f1.getPrivateKeys = f2.getPrivateKeys = gpk
-        f1.startFactory()
-        f2.startFactory()
+    if not pyasn1:
+        skip = "can't run w/o PyASN1"
+
+    def makeSSHFactory(self, primes=None):
+        sshFactory = factory.SSHFactory()
+        gpk = lambda: {'ssh-rsa' : keys.Key(None)}
+        sshFactory.getPrimes = lambda: primes
+        sshFactory.getPublicKeys = sshFactory.getPrivateKeys = gpk
+        sshFactory.startFactory()
+        return sshFactory
+
+
+    def test_buildProtocol(self):
+        """
+        By default, buildProtocol() constructs an instance of
+        SSHServerTransport.
+        """
+        factory = self.makeSSHFactory()
+        protocol = factory.buildProtocol(None)
+        self.assertIsInstance(protocol, transport.SSHServerTransport)
+
+
+    def test_buildProtocolRespectsProtocol(self):
+        """
+        buildProtocol() calls 'self.protocol()' to construct a protocol
+        instance.
+        """
+        calls = []
+        def makeProtocol(*args):
+            calls.append(args)
+            return transport.SSHServerTransport()
+        factory = self.makeSSHFactory()
+        factory.protocol = makeProtocol
+        factory.buildProtocol(None)
+        self.assertEquals([()], calls)
+
+
+    def test_multipleFactories(self):
+        f1 = self.makeSSHFactory(primes=None)
+        f2 = self.makeSSHFactory(primes={1:(2,3)})
         p1 = f1.buildProtocol(None)
         p2 = f2.buildProtocol(None)
         self.failIf('diffie-hellman-group-exchange-sha1' in p1.supportedKeyExchanges,
                 p1.supportedKeyExchanges)
         self.failUnless('diffie-hellman-group-exchange-sha1' in p2.supportedKeyExchanges,
                 p2.supportedKeyExchanges)
+
+
+
+class MPTestCase(unittest.TestCase):
+    """
+    Tests for L{common.getMP}.
+
+    @cvar getMP: a method providing a MP parser.
+    @type getMP: C{callable}
+    """
+    getMP = staticmethod(common.getMP)
+
+    if not Crypto:
+        skip = "can't run w/o PyCrypto"
+
+    if not pyasn1:
+        skip = "can't run w/o PyASN1"
+
+
+    def test_getMP(self):
+        """
+        L{common.getMP} should parse the a multiple precision integer from a
+        string: a 4-byte length followed by length bytes of the integer.
+        """
+        self.assertEquals(
+            self.getMP('\x00\x00\x00\x04\x00\x00\x00\x01'),
+            (1, ''))
+
+
+    def test_getMPBigInteger(self):
+        """
+        L{common.getMP} should be able to parse a big enough integer
+        (that doesn't fit on one byte).
+        """
+        self.assertEquals(
+            self.getMP('\x00\x00\x00\x04\x01\x02\x03\x04'),
+            (16909060, ''))
+
+
+    def test_multipleGetMP(self):
+        """
+        L{common.getMP} has the ability to parse multiple integer in the same
+        string.
+        """
+        self.assertEquals(
+            self.getMP('\x00\x00\x00\x04\x00\x00\x00\x01'
+                       '\x00\x00\x00\x04\x00\x00\x00\x02', 2),
+            (1, 2, ''))
+
+
+    def test_getMPRemainingData(self):
+        """
+        When more data than needed is sent to L{common.getMP}, it should return
+        the remaining data.
+        """
+        self.assertEquals(
+            self.getMP('\x00\x00\x00\x04\x00\x00\x00\x01foo'),
+            (1, 'foo'))
+
+
+    def test_notEnoughData(self):
+        """
+        When the string passed to L{common.getMP} doesn't even make 5 bytes,
+        it should raise a L{struct.error}.
+        """
+        self.assertRaises(struct.error, self.getMP, '\x02\x00')
+
+
+
+class PyMPTestCase(MPTestCase):
+    """
+    Tests for the python implementation of L{common.getMP}.
+    """
+    getMP = staticmethod(common.getMP_py)
+
+
+
+class GMPYMPTestCase(MPTestCase):
+    """
+    Tests for the gmpy implementation of L{common.getMP}.
+    """
+    getMP = staticmethod(common._fastgetMP)
+
+
+
+try:
+    import gmpy
+except ImportError:
+    GMPYMPTestCase.skip = "gmpy not available"
